package com.dynamixsoftware.intentapi;

import com.dynamixsoftware.intentapi.IJob;
import com.dynamixsoftware.intentapi.IPrinterContext;
import android.graphics.Bitmap;
import android.graphics.Rect;

/**
 * Interface is used for document description.
 */
interface IDocument {
	
	/**
	 * Initializes printers context. It is called every time settings are changed.
	 * @param printerContext printers context
	 * @param thumbnailWidth Maximum thumbnail width in pixels 
	 * @param thumbnailHeight Maximum thumbnail height in pixels
	 */
	void initDeviceContext(IPrinterContext printerContext, int thumbnailWidth, int thumbnailHeight);
	
	/**
	 * Shows text for user(usually document name, document path, etc)
	 * @return description string
	 */
	String getDescription();
		
	/**
	 * Gets page thumbnail. Application goes to infinite loop if 
	 * OutOfMemoryError happens. It calls  initDeviceContext() 
	 * with less thumbnail width and heigth. Null can be returned to stop loop.
	 * This page thumbnail won't be shown.
	 * @param pageNum page number
	 * @return bitmap of the thumbnail or null
	 */
	Bitmap getPageThumbnail(int pageNum);
	
	/**
	 * Returns total pages
	 * @return number of pages
	 */
	int getTotalPages();

	/**
	 * Gets page fragment.
	 * @param num page number
	 * @param fragment rect of the fragment
	 * @return bitmap of the fragment
	 */
	Bitmap renderPageFragment(int num, in Rect fragment);
	
}
