package com.dynamixsoftware.intentapi;

import com.dynamixsoftware.intentapi.IPrinterInfo;
import com.dynamixsoftware.intentapi.IDocument;
import com.dynamixsoftware.intentapi.IJob;
import com.dynamixsoftware.intentapi.IPrintCallback;
import com.dynamixsoftware.intentapi.IServiceCallback;
import com.dynamixsoftware.intentapi.ISetLicenseCallback;
import com.dynamixsoftware.intentapi.PrinterOption;
import com.dynamixsoftware.intentapi.PrintHandOption;

interface IIntentAPI {

	/**
	 * Tries to get current printer.
	 * @return current printer info or null.
	 */
	IPrinterInfo getCurrentPrinter();
	
	/**
	 * Starts printing on current printer
	 * @param job name
	 * @param job to be printed
	 * @param copies number
	 * @return true if printing ended successfully
	 */
	boolean print(String jobName, IJob job, int copies);
	
	/**
	 * Callback can be set for control printing process. 
	 * @param printCallback to be set.
	 */
	void setPrintCallback(IPrintCallback printCallback);
	
	/**
	 * Callback can be set for control service processes. 
	 * @param serviceCallback to be set.
	 */
	void setServiceCallback(IServiceCallback serviceCallback);
	
	/**
	 * Sets IDocument for printing. 
	 * @param printCallback to be set.
	 */
	void setDocument(IDocument document);
	
	void setLicense(String id, ISetLicenseCallback licenseCallback);
	
	List<String> getPrintJobs();
	
	void removePrintJob(String printJobName);
	
	void printPHrendering(String jobName, String mimeType, in Uri uri);
	
	List<PrintHandOption> getImagesOptions();
	
	List<PrintHandOption> getFilesOptions();
	
	void setImagesOptions(in List<PrintHandOption> options);
	
	void setFilesOptions(in List<PrintHandOption>options);

}
