package com.dynamixsoftware.intentapi;

import com.dynamixsoftware.intentapi.Result;

/**
 * Interface is used for printing callbacks.
 */
interface IPrintCallback {
	
	/**
	 * Start of the printing.
	 */
	void start();
	
	/**
	 * Actual start of the print job.
	 */
	void startingPrintJob();
	
	/**
	 * Preparing page
	 * @param num page number
	 */
	void preparePage(int pageNum);
	
	/**
	 * Sending page
	 * @param num page number
	 * @param progress printing progress in percents
	 */
	void sendingPage(int pageNum, int progress);
	
	/**
	 * @return true to cancel print job, false otherwise.
	 */
	boolean needCancel();
	
	/**
	 * Finishing print job.
	 */
	void finishingPrintJob();
	
	/**
	 * Finish of printing.
	 * @param result number of pages that were printed.
	 * @param pagesPrinted number of pages that were printed.
	 */
	void finish(in Result result, int pagesPrinted);

}
