package com.dynamixsoftware.intentapi;

import com.dynamixsoftware.intentapi.IPrinterContext;
import com.dynamixsoftware.intentapi.PrinterOption;
import com.dynamixsoftware.intentapi.PrinterOptionValue;

/**
 * Interface describes printers info.
 */
interface IPrinterInfo {

	/**
	 * @return printer type.
	 * wi-fi == 0
	 * bluetooth == 1
	 * cloud == 2
	 * smb == 3
	 * usb == 4
	 * PrintHand client ==5
	 */
	int getType();
	
	/**
	 * @return printer owner.
	 */
	String getOwner();
	
	/**
	 * @return printer name.
	 */
	String getName();
	
	/**
	 * @return printer description.
	 */
	String getDescription();
	
	/**
	 * @return printer context.
	 */
	IPrinterContext getPrinterContext();
	
	/**
	 * @return printers option list.
	 */
	List<PrinterOption> getOptions();
	
	/** 
	 * @return current option value.
	 */
	PrinterOptionValue getOptionValue(in PrinterOption option);
	
	/** 
	 * @return printers option values list.
	 */
	List<PrinterOptionValue> getOptionValueList(in PrinterOption option);
	
	/**
	 * Changes option value
	 * option to be changed
	 * value to be set
	 * @return true if printers context changed, false otherwise
	 */
	boolean setOption(in PrinterOption option, in PrinterOptionValue value);
	
}