package com.dynamixsoftware.printingsdk;

import com.dynamixsoftware.printingsdk.ISetupPrinterListener;
import com.dynamixsoftware.printingsdk.Printer;
import com.dynamixsoftware.printingsdk.PrinterOption;
import com.dynamixsoftware.printingsdk.PrinterOptionValue;
import com.dynamixsoftware.printingsdk.IDiscoverListener;
import com.dynamixsoftware.printingsdk.IDiscoverCloudListener;
import com.dynamixsoftware.printingsdk.IDiscoverSmb;
import com.dynamixsoftware.printingsdk.IDiscoverSmbListener;
import com.dynamixsoftware.printingsdk.IFindDriversListener;
import com.dynamixsoftware.printingsdk.ISetupPrinterListener;
import com.dynamixsoftware.printingsdk.DriverHandle;
import com.dynamixsoftware.printingsdk.TransportType;
import com.dynamixsoftware.printingsdk.IGetDriversListener;
import com.dynamixsoftware.printingsdk.IPrintListener;
import com.dynamixsoftware.printingsdk.Page;
import com.dynamixsoftware.printingsdk.IPrintPage;
import com.dynamixsoftware.printingsdk.ISetLicenseCallback;

interface IPrintingSdk {
	
	void initRecentPrinters(ISetupPrinterListener setupPrinterListener);
	Printer getCurrentPrinter();
	List<Printer> getRecentPrintersList();
	
	/**
	 * Changes option value
	 * option to be changed
	 * value to be set
	 * @return true if printers context changed, false otherwise
	 */
	boolean setCurrentPrinterOption(in PrinterOption option, in PrinterOptionValue value);
	
	/**
	 * Starts Wi-Fi printers discovering.
	 * 
	 * @param discoverCallback reports discover status.
	 * @return false if discover already runs or bluetooth discover active. True if discover can be launched.
	 */
	boolean startDiscoverWiFi(IDiscoverListener discoverListener);
	
	/**
	 * Starts Wi-Fi printers discovering with lower timeout.
	 * 
	 * @param discoverListener reports discover status.
	 * @return false if discover already runs or bluetooth discover active. True if discover can be launched.
	 */
	boolean startQuickDiscoverWiFi(IDiscoverListener discoverListener);
	
	/**
	 * Tries to stop Wi-Fi discovering.
	 */
	void stopDiscoverWiFi();
	
	/**
	 * Starts Google Cloud printers discovering.
	 * 
	 * @param account name of Google account.
	 * @param discoverCloudCallback reports discover status.
	 */
	 void startDiscoverCloud(String account, IDiscoverCloudListener discoverCloudListener);
	 
	 /**
	 * Starts bluetooth printers discovering.
	 * 
	 * @param discoverCallback reports discover status.
	 * @return false if discover already runs or Wi-Fi discover active. True if
	 *         discover can be launched.
	 */
	 boolean startDiscoverBluetooth(IDiscoverListener discoverListener);
	 
	 /**
	 * Tries to stop bluetooth discovering.
	 */
	 void stopDiscoverBluetooth();
	 
	 /**
	 * Starts SMB printers discovering.
	 * @param discoverSmbCallback reports discover status.
	 * @return discover interface
	 */
	 IDiscoverSmb startDiscoverSmb(IDiscoverSmbListener discoverSmbListener);
	 
	 /**
	 * Starts USB discovering
	 * @param discoverCallback reports discover status.
	 */
	 void startDiscoverUSB(IDiscoverListener discoverListener);
	 
	 /**
	 * Tries to find most appropriate drivers for printer.
	 * 
	 * @param printer For this printer drivers will be searched.
	 * @param findDriversCallback reports drivers search status.
	 */
	 void findDrivers(in Printer printer, IFindDriversListener findDriversListener);
	 
	 void getDriversList(in Printer printer, in TransportType transportType, IGetDriversListener getDriversListener);
	 
	 /**
	 * Setup recent printer.
	 * @param printer to setup.
	 * @param forceInstall true if library should be installed, false otherwise.
	 * @param callback setup callback.
	 */
	 void setupRecent(in Printer printer, boolean forceInstall, ISetupPrinterListener setupListener);
	 
	 /**
	 * Setup discovered printer.
	 * @param printer to setup.
	 * @param driverHandle to setup with.
	 * @param transportType to setup with.
	 * @param forceInstall true if library should be installed, false otherwise.
	 * @param callback setup callback.
	 */
	 void setup(in Printer printer, in DriverHandle driverHandle, in TransportType transportType, boolean forceInstall, ISetupPrinterListener setupListener);
	 
	 void print(int pagesNum, int copies, IPrintPage printPage, IPrintListener printListener);
	 
	 void setLicense(String id, ISetLicenseCallback licenseCallback);
	
}
